<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once "BaseController.php";
require_once "SearchTrail.php";
require_once "WSConstants.php";
require_once "Paginator.php";
require_once 'Logger.php';
require_once 'InQuireWebUtil.php';
require_once 'ConfigFileManagerModel.php';

/**
 * FilterController is used for filtering search results. it is responsible for filtering following
 * actions.
 *
 * <li>Suggested Keyword Action</li>
 * <li>Keyword Filter Action</li>
 * <li>Category Filter Action</li>
 * <li>Manufacturer Filter Action</li>
 * <li>Attribute Filter Action</li>
 *
 * FilterController also manages the searc and display trail of user filtered searches.
 *
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 *
 * @see FilterController
 * @see SearchTrail
 */
class FilterController extends BaseController
{

	/**
	 * suggestAction is used to perform search with suggested keyword.
	 *
	 * It initialize a new SearchTrail for every suggested search and gets suggested keywords and searched keyword from GET parameters by using
	 * 'suggestedKeyword' and 'keyword' as the parameter name.
	 *
	 * Newly initiated SearchTrail is then set in the REQUEST as a string with parameter name 'trail'. 'trail' parameter is then used in the view
	 * to generate the display search trail.
	 *
	 * This action uses web service model classes to perform the web services requests. WSFactory is used to create different keyword search requests.
	 * WSRequest is used to encapsulate the search keywords. WSRequest instance is than passed to WSGateway which passed send request to spexlive
	 * client and finally request is send to http://ws.spexlive.net.
	 *
	 * WSResponse object is created for the response of http://ws.spexlive.net. WSResponse object contains the XML data returned from hosted
	 * web services. This XML data is converted into a php object and then forwarded to view. Successfull response is forwarded to view that generate
	 * different filters and search results table with pagination.
	 *
	 * For any reason web service call is failed (even after retries) than request is forwarded to the ErrorController.
	 *
	 * Following objects are set for the view:
	 *
	 * <li>categoryFilter, It is used to create categories filter.</li>
	 * <li>manufacturerFilter, It is used to create manufacturers filter.</li>
	 * <li>attributeFilter, It is used to create attributes filter.</li>
	 * <li>trail, It is used to display search trail.</li>
	 * <li>productSummary, It is used to display product summarized information.</li>
	 * <li>page, It is used to display page no.</li>
	 * <li>count, It is used to display result count.</li>
	 *
	 *
	 * @access public
	 * @param keyword, this is a request parameter and is used to get keyword query string
	 * @param suggestedKeyword, this is a request parameter and is used to get suggested keyword query string
	 * @param trail, this is a request parameter and is set to store the search trail string
	 * @return it doesn't return anything
	 *
	 * @see SearchTrail
	 * @see WSRequest
	 * @see WSFactory
	 * @see WSGateway
	 * @see WSResponse
	 * @see ErrorController
	 * @see Paginator
	 *
	 */
	public function suggestAction ()
	{
		/**
		 * Get keyword from request parameter
		 */
		$keyword = $_REQUEST["keyword"];
		/**
		 * Get suggested keyword from request parameter
		 */
		$suggestion = trim($_GET['suggestedKeyword']);
        $suggestion = str_replace("%2F", "\\", $suggestion);
        $suggestion = str_replace('\"', '"', $suggestion);
        $suggestion = str_replace("\'", "'", $suggestion);
        
        $suggestion = str_replace(" ", "+", $suggestion);
        
		//Display user Selected Keyword on header
		$_SESSION["header"] = $suggestion;
		/**
		 * Set trialString from request.
		 */
		//$trailString = $_REQUEST["trail"];
		/**
		 * Create new search trial with previous search.
		 */
		//$trail = new SearchTrail($trailString);
		/**
		 * Create new search trial.
		 */
		$trail = new SearchTrail("");
		/**
		 * Remove original keyword from SearchTrail
		 */
		//$trail->remove($keyword, "kw");
		/**
		 * Add keyword filter in SearchTrail
		 */
		$trail->addKeyword($suggestion);
		/**
		 * Set trail string in request.
		 */
		$_REQUEST["trail"] = $trail->getTrialString();
		/**
		 * Prepare paramters for WSRequest
		 */
		$parameters = $this->setSearchParameters();
		/**
		 * String contains url parameters directly appeded to WSRequest url.
		 */
		$parameters['appender'] = $trail->toParameterString("");
		/**
		 * Array for extra parameters merge with original WSRequest url.
		 */
		$params['suggestKeywordPharases'] = 'true';
		$parameters['plugArray'] = $params;
		/**
		 * Call WebServices Gateway and return WSResponse.
		 */
		Logger::info("suggestAction Calling WS");

		$phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);
		if ($phpobject !== false) {
			/*
			 * Check  the returned xml contents for Error message
			 */
			if(InQuireWebUtil::isErrorResponse($phpobject)){
				$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
				return;
			}
			 
			/**
			 * Check if search results retrun zero product then remove actual search from trail.
			 */
			if ($phpobject['count'] == 0) { //->$trail->remove($suggestion, "kw");
				//->$_REQUEST["trail"] = $trail->getTrialString();
			}
			/**
			 * create the pagination table
			 */
			$paginator = new Paginator(urlencode($trail->getTrialString()));
			$paginator->load(range(0, $phpobject['count'] - 1), $_SESSION["pageSize"], 1);
			/**
			 * set view to create page navigations
			 */
			$this->view->paginate = $paginator->getNavigation();
			/**
			 * set views for search resutls template
			 */
			$searchResultViewObjectsStatus = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);

			//See if Web Service returned error response and redirected to error page
			if($searchResultViewObjectsStatus == WSConstants::$ERROR_RESPONSE){
				return ;
			}
			/**
			 * set the view and move on
			 */
			$this->_request->setControllerName("search")->setActionName("searchresults");
		} else {
			$this->setWSRequestFailed();
		}
	}


	/**
	 * keywordAction is used to filter search based on the keyword entered.
	 *
	 * It get the SearchTrail obejct from previous request and add the selected keyword in trail.
	 * 
	 * keyword retrive from GET parameters by using 'keywords' as the parameter name.
	 * 
	 * Modified SearchTrail with filtered search is then set in the REQUEST as a string with parameter 
	 * name 'trail'. 'trail' parameter is then used in the view to generate the display search trail.
	 * 
	 * DisplaySearch trail list the selected filters.
	 *
	 * This action uses web service model classes to perform the web services requests. WSFactory is used to create different keyword search requests.
	 * WSRequest is used to encapsulate the search keywords. WSRequest instance is than passed to WSGateway which passed send request to spexlive
	 * client and finally request is send to http://ws.spexlive.net.
	 *
	 * WSResponse object is created for the response of http://ws.spexlive.net. WSResponse object contains the XML data returned from hosted
	 * web services. This XML data is converted into a php object and then forwarded to view. Successfull response is forwarded to view that generate
	 * different filters and search results table with pagination.
	 *
	 * For any reason web service call is failed (even after retries) than request is forwarded to the ErrorController.
	 *
	 * Following objects are set for the view depending on the filtered search results:
	 *
	 * <li>categoryFilter, It is used to create categories filter.</li>
	 * <li>manufacturerFilter, It is used to create manufacturers filter.</li>
	 * <li>attributeFilter, It is used to create attributes filter.</li>
	 * <li>trail, It is used to display search trail.</li>
	 * <li>productSummary, It is used to display product summarized information.</li>
	 * <li>page, It is used to display page no.</li>
	 * <li>count, It is used to display result count.</li>
	 *
	 * @see SearchTrail
	 * @see WSRequest
	 * @see WSFactory
	 * @see WSGateway
	 * @see WSResponse
	 * @see ErrorController
	 * @see Paginator
	 */
	public function keywordAction ()
	{
		/**
		 * Get keyword from request parameter
		 */
		$keyword = trim($_REQUEST["keywords"]);
		$keyword = str_replace("%2F", "\\", $keyword);
		$keyword = str_replace('\"', '"', $keyword);
		$keyword = str_replace("\'", "'", $keyword);
		/**
		 * Replace every space in keyword with "+" sign
		 */
		$keyword = str_replace(" ", "+", $keyword);
		/**
		 * Set trialString from request.
		 */
		$trailString = urldecode($_REQUEST["trail"]);
		/**
		 * Create new search trial with previous search.
		 */
		$trail = new SearchTrail($trailString);
		/**
		 * Add keyword filter in SearchTrail
		 */
		$trail->addKeyword($keyword);
		/**
		 * Set trail string in request.
		 */
		$_REQUEST["trail"] = $trail->getTrialString();
		/**
		 * Prepare paramters for WSRequest
		 */
		$parameters = $this->setSearchParameters();
		/**
		 * String contains url parameters directly appeded to WSRequest url.
		 */
		$parameters['appender'] = $trail->toParameterString("");
		/**
		 * Array for extra parameters merge with original WSRequest url.
		 */
		$params['suggestKeyword'] = 'true';
		$parameters['plugArray'] = $params;
		/**
		 * Call WebServices Gateway and return WSResponse.
		 */
		Logger::info("keywordAction Calling WS");

		$phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);
		if ($phpobject !== false) {
			/*
			 * Check  the returned xml contents for Error message
			 */
			if(InQuireWebUtil::isErrorResponse($phpobject)){
				$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
				return;
			}
			 

			/**
			 * create the pagination table
			 */
			$paginator = new Paginator(urlencode($trail->getTrialString()));
			$paginator->load(range(0, $phpobject['count'] - 1), $_SESSION["pageSize"], 1);
			/**
			 * set view to create page navigations
			 */
			$this->view->paginate = $paginator->getNavigation();
			/**
			 * set views for search resutls template
			 */
			$searchResultViewObjectsStatus = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);

			//See if Web Service returned error response and redirected to error page
			if($searchResultViewObjectsStatus == WSConstants::$ERROR_RESPONSE){
				return ;
			}

			/**
			 * set the view and move on
			 */
			$this->_request->setControllerName("search")->setActionName("searchresults");
		} else {
			$this->setWSRequestFailed();
		}
	}

	/**
	 * categoryAction is used to filter search based on the category selected.
	 *
	 * It get the SearchTrail obejct from previous request and add the selected category in trail.
	 * 
	 * Cateogry Id retrive from GET parameters by using 'id' as the parameter name.
	 * 
	 * Modified SearchTrail with filtered search is then set in the REQUEST as a string with parameter 
	 * name 'trail'. 'trail' parameter is then used in the view to generate the display search trail.
	 * 
	 * DisplaySearch trail list the selected filters.
	 *
	 * This action uses web service model classes to perform the web services requests. WSFactory is used to create different keyword search requests.
	 * WSRequest is used to encapsulate the search keywords. WSRequest instance is than passed to WSGateway which passed send request to spexlive
	 * client and finally request is send to http://ws.spexlive.net.
	 *
	 * WSResponse object is created for the response of http://ws.spexlive.net. WSResponse object contains the XML data returned from hosted
	 * web services. This XML data is converted into a php object and then forwarded to view. Successfull response is forwarded to view that generate
	 * different filters and search results table with pagination.
	 *
	 * For any reason web service call is failed (even after retries) than request is forwarded to the ErrorController.
	 *
	 * Following objects are set for the view depending on the filtered search results:
	 *
	 * <li>categoryFilter, It is used to create categories filter.</li>
	 * <li>manufacturerFilter, It is used to create manufacturers filter.</li>
	 * <li>attributeFilter, It is used to create attributes filter.</li>
	 * <li>trail, It is used to display search trail.</li>
	 * <li>productSummary, It is used to display product summarized information.</li>
	 * <li>page, It is used to display page no.</li>
	 * <li>count, It is used to display result count.</li>
	 *
	 * @see SearchTrail
	 * @see WSRequest
	 * @see WSFactory
	 * @see WSGateway
	 * @see WSResponse
	 * @see ErrorController
	 * @see Paginator
	 */	
	public function categoryAction ()
	{
		$config = ConfigFileManagerModel::getInstance();
		$categoryType = $config->getValue('category', 'inQuire');
		
		/**
		 * Get category id and value from request.
		 */
		$categoryId = $_GET["id"] . '$' . $categoryType;
		/**
		 * Get trail from request
		 */
		$trailString = $_GET["trail"];
		$trailString = str_replace("%26", "&", $trailString);
		$trailString = str_replace("%3D", "=", $trailString);
		$trailString = str_replace("%20", " ", $trailString);
		/**
		 * Create new SearchTrail from previous search.
		 */
		$trail = new SearchTrail($trailString);
		/**
		 * Add category in search trail.
		 */
		$trail->addCategory($categoryId);
		$_REQUEST["trail"] = $trail->getTrialString();
		/**
		 * Prepare paramters for WSRequest
		 */
		$parameters = $this->setSearchParameters();
		/**
		 * String contains url parameters directly appeded to WSRequest url.
		 */
		$parameters['appender'] = $trail->toParameterString("category");
		/**
		 * Call WebServices Gateway and return WSResponse.
		 */
		Logger::info("categoryAction Calling WS");
		$phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);
		if ($phpobject !== false) {
			/*
			 * Check  the returned xml contents for Error message
			 */
			if(InQuireWebUtil::isErrorResponse($phpobject)){
				$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
				return;
			}
			 
			/**
			 * create the pagination table
			 */
			$paginator = new Paginator(urlencode($trail->getTrialString()));
			$paginator->load(range(0, $phpobject['count'] - 1), $_SESSION["pageSize"], 1);
			/**
			 * set view to create page navigations
			 */
			$this->view->paginate = $paginator->getNavigation();
			/**
			 * set views for search resutls template
			 */
			$searchResultViewObjectsStatus = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);

			//See if Web Service returned error response and redirected to error page
			if($searchResultViewObjectsStatus == WSConstants::$ERROR_RESPONSE){
				return ;
			}
			/**
			 * set the view and move on
			 */
			$this->_request->setControllerName("search")->setActionName("searchresults");
		} else {
			$this->setWSRequestFailed();
		}
	}

	/**
	 * manufacturerAction is used to filter search based on the manufacturer selected.
	 *
	 * It get the SearchTrail obejct from previous request and add the selected manufacturer in trail.
	 * 
	 * Manufacturer Id retrive from GET parameters by using 'id' as the parameter name.
	 * 
	 * Modified SearchTrail with filtered search is then set in the REQUEST as a string with parameter 
	 * name 'trail'. 'trail' parameter is then used in the view to generate the display search trail.
	 * 
	 * DisplaySearch trail list the selected filters.
	 *
	 * This action uses web service model classes to perform the web services requests. WSFactory is used to create different keyword search requests.
	 * WSRequest is used to encapsulate the search keywords. WSRequest instance is than passed to WSGateway which passed send request to spexlive
	 * client and finally request is send to http://ws.spexlive.net.
	 *
	 * WSResponse object is created for the response of http://ws.spexlive.net. WSResponse object contains the XML data returned from hosted
	 * web services. This XML data is converted into a php object and then forwarded to view. Successfull response is forwarded to view that generate
	 * different filters and search results table with pagination.
	 *
	 * For any reason web service call is failed (even after retries) than request is forwarded to the ErrorController.
	 *
	 * Following objects are set for the view depending on the filtered search results:
	 *
	 * <li>categoryFilter, It is used to create categories filter.</li>
	 * <li>manufacturerFilter, It is used to create manufacturers filter.</li>
	 * <li>attributeFilter, It is used to create attributes filter.</li>
	 * <li>trail, It is used to display search trail.</li>
	 * <li>productSummary, It is used to display product summarized information.</li>
	 * <li>page, It is used to display page no.</li>
	 * <li>count, It is used to display result count.</li>
	 *
	 * @see SearchTrail
	 * @see WSRequest
	 * @see WSFactory
	 * @see WSGateway
	 * @see WSResponse
	 * @see ErrorController
	 * @see Paginator
	 */	
	public function manufacturerAction ()
	{
		/**
		 * Get manufacturer id and value from request parameters.
		 */
		$mfgId = $_GET["id"];
		$trailString = $_GET["trail"];
		/**
		 * Create new search trial from previous search.
		 */
		$trail = new SearchTrail($trailString);
		$trail->addManufacturer($mfgId);
		$_REQUEST["trail"] = $trail->getTrialString();
		/**
		 * Prepare paramters for WSRequest
		 */
		$parameters = $this->setSearchParameters();
		/**
		 * String contains url parameters directly appeded to WSRequest url.
		 */
		$parameters['appender'] = $trail->toParameterString(""); //
		Logger::info("manufacturerAction Calling WS");

		$phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);
		if ($phpobject !== false) {
			/*
			 * Check  the returned xml contents for Error message
			 */
			if(InQuireWebUtil::isErrorResponse($phpobject)){
				$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
				return;
			}
			 
			/**
			 * create the pagination table
			 */
			$paginator = new Paginator(urlencode($trail->getTrialString()));
			$paginator->load(range(0, $phpobject['count'] - 1), $_SESSION["pageSize"], 1);
			/**
			 * set view to create page navigations
			 */
			$this->view->paginate = $paginator->getNavigation();
			/**
			 * set views for search resutls template
			 */
			$searchResultViewObjectsStatus = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);

			//See if Web Service returned error response and redirected to error page
			if($searchResultViewObjectsStatus == WSConstants::$ERROR_RESPONSE){
				return ;
			}

			/**
			 * set the view and move on
			 */
			$this->_request->setControllerName("search")->setActionName("searchresults");
		} else {
			$this->setWSRequestFailed();
		}
	}

	/**
	 * attributeAction is used to filter search based on the attribute selected.
	 * attributeAction provide support to filter absolute and non-absolute attributes.
	 * 
	 * It get the SearchTrail obejct from previous request and add the selected attribute in trail.
	 * 
	 * Attribute data retrieve from GET parameters by using 'attributefilter' as the parameter name.
	 * It contians concatenated string with attribtue id and value. If select attribute us absolute then
	 * 'absoluteOperator' also retrieve from GET parameters list.	 * 
	 * 
	 * Modified SearchTrail with filtered search is then set in the REQUEST as a string with parameter 
	 * name 'trail'. 'trail' parameter is then used in the view to generate the display search trail.
	 * 
	 * DisplaySearch trail list the selected filters.
	 *
	 * This action uses web service model classes to perform the web services requests. WSFactory is used to create different keyword search requests.
	 * WSRequest is used to encapsulate the search keywords. WSRequest instance is than passed to WSGateway which passed send request to spexlive
	 * client and finally request is send to http://ws.spexlive.net.
	 *
	 * WSResponse object is created for the response of http://ws.spexlive.net. WSResponse object contains the XML data returned from hosted
	 * web services. This XML data is converted into a php object and then forwarded to view. Successfull response is forwarded to view that generate
	 * different filters and search results table with pagination.
	 *
	 * For any reason web service call is failed (even after retries) than request is forwarded to the ErrorController.
	 *
	 * Following objects are set for the view depending on the filtered search results:
	 *
	 * <li>categoryFilter, It is used to create categories filter.</li>
	 * <li>manufacturerFilter, It is used to create manufacturers filter.</li>
	 * <li>attributeFilter, It is used to create attributes filter.</li>
	 * <li>trail, It is used to display search trail.</li>
	 * <li>productSummary, It is used to display product summarized information.</li>
	 * <li>page, It is used to display page no.</li>
	 * <li>count, It is used to display result count.</li>
	 *
	 * @see SearchTrail
	 * @see WSRequest
	 * @see WSFactory
	 * @see WSGateway
	 * @see WSResponse
	 * @see ErrorController
	 * @see Paginator
	 */	
	public function attributeAction ()
	{
		/**
		 * Get attribute filter value from request.
		 */
		$attributefilter = urlencode($_REQUEST["attributefilter"]);
		$attributefilter = str_replace("%26", "&", "$attributefilter");
		$attributefilter = str_replace("%3D", "=", "$attributefilter");
		$addfilter = urldecode($attributefilter);
		/**
		 * Split attribute in id and value.
		 */
		$attribute = explode("|", $addfilter);
		$attrId = $attribute[0];
		$attrValue = $attribute[1];
		$trail = new SearchTrail(urldecode($_REQUEST["trail"]));
		/**
		 * Get absolute oprator from request
		 */		
		if (array_key_exists('absoluteOperator', $_REQUEST)) {
			$absOptr = $_REQUEST["absoluteOperator"];
			
			if ($absOptr > - 1) {
				if (! $absOptr)
					$absOptr = 0;
				/*
				$attribute = explode("|", $attribute[1]);
				$attrValue = $attribute[0];
				$attrDisplayValue = urldecode($attribute[1]);
				$attrDisplayValue = str_replace('`', '"', $attrDisplayValue);
				*/
				$trail->addAbsoluteAttribute($attrId, $attrValue, $absOptr);
			}
		} else {
			$attrValue = str_replace("~", "'", $attrValue);
			$trail->addAttribute($attrId, $attrValue);
		}
		$_REQUEST["trail"] = $trail->getTrialString();
		/**
		 * Prepare paramters for WSRequest
		 */
		$parameters = $this->setSearchParameters();
		/**
		 * String contains url parameters directly appeded to WSRequest url.
		 */
		$parameters['appender'] = $trail->toParameterString("");
		Logger::info("attributeAction Calling WS");
		
		$phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);
		if ($phpobject !== false) {
			 
			/*
			 * Check  the returned xml contents for Error message
			 */
			if(InQuireWebUtil::isErrorResponse($phpobject)){
				$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
				return;
			}

			/**
			 * Check if search results retrun zero product then remove actual search from trail.
			 */
			if ($phpobject['count'] == 0) { //->$trail->remove($attrId, "attributeFilter");
				//->$_REQUEST["trail"] = $trail->getTrialString();
			}
			/**
			 * create the pagination table
			 */
			$paginator = new Paginator(urlencode($trail->getTrialString()));
			$paginator->load(range(0, $phpobject['count'] - 1), $_SESSION["pageSize"], 1);
			/**
			 * set view to create page navigations
			 */
			$this->view->paginate = $paginator->getNavigation();
			/**
			 * set views for search resutls template
			 */
			$searchResultViewObjectsStatus = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);

			//See if Web Service returned error response and redirected to error page
			if($searchResultViewObjectsStatus == WSConstants::$ERROR_RESPONSE){
				return ;
			}
			/**
			 * set the view and move on
			 */
			$this->_request->setControllerName("search")->setActionName("searchresults");
		} else {
			$this->setWSRequestFailed();
		}
	}

	/**
	 * removeAction is used to remove selected filter from search based on the id and filter passed in
	 * request paramters.
	 *  
	 * It get the SearchTrail obejct from previous request and remove the selected filter from trail.
	 * 
	 * Modified SearchTrail with filtered search is then set in the REQUEST as a string with parameter 
	 * name 'trail'. 'trail' parameter is then used in the view to generate the display search trail.
	 * 
	 * DisplaySearch trail list the selected filters.
	 *
	 * This action uses web service model classes to perform the web services requests. WSFactory is used to create different keyword search requests.
	 * WSRequest is used to encapsulate the search keywords. WSRequest instance is than passed to WSGateway which passed send request to spexlive
	 * client and finally request is send to http://ws.spexlive.net.
	 *
	 * WSResponse object is created for the response of http://ws.spexlive.net. WSResponse object contains the XML data returned from hosted
	 * web services. This XML data is converted into a php object and then forwarded to view. Successfull response is forwarded to view that generate
	 * different filters and search results table with pagination.
	 *
	 * For any reason web service call is failed (even after retries) than request is forwarded to the ErrorController.
	 *
	 * Following objects are set for the view depending on the filtered search results:
	 *
	 * <li>categoryFilter, It is used to create categories filter.</li>
	 * <li>manufacturerFilter, It is used to create manufacturers filter.</li>
	 * <li>attributeFilter, It is used to create attributes filter.</li>
	 * <li>trail, It is used to display search trail.</li>
	 * <li>productSummary, It is used to display product summarized information.</li>
	 * <li>page, It is used to display page no.</li>
	 * <li>count, It is used to display result count.</li>
	 *
	 * @see SearchTrail
	 * @see WSRequest
	 * @see WSFactory
	 * @see WSGateway
	 * @see WSResponse
	 * @see ErrorController
	 * @see Paginator
	 */
	public function removeAction ()
	{
		$elementId = $_GET["id"];
		$elementType = urldecode($_GET["type"]);
		$trail = new SearchTrail($_GET["trail"]);
		if ($elementType == "parentcategoryFilterId") {
			$pcIds = explode("|", $elementId);
			$trail->remove($pcIds[0], $elementType);
			for ($i = 0; $i < sizeof($pcIds); $i ++) {
				$trail->remove($pcIds[$i], "parentcategoryFilterId");
			}
		} else {
			$trail->remove($elementId, $elementType);
		}
		$_REQUEST["trail"] = $trail->getTrialString();
		/**
		 * Prepare paramters for WSRequest
		 */
		$parameters = $this->setSearchParameters();
		/**
		 * String contains url parameters directly appeded to WSRequest url.
		 */
		$parameters['appender'] = $trail->toParameterString(""); //
		Logger::info("removeAction Calling WS");

		$phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);
		if ($phpobject !== false) {

			/*
			 * Check  the returned xml contents for Error message
			 */
			if(InQuireWebUtil::isErrorResponse($phpobject)){
				$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
				return;
			}

			/**
			 * create the pagination table
			 */
			$paginator = new Paginator(urlencode($trail->getTrialString()));
			$paginator->load(range(0, $phpobject['count'] - 1), $_SESSION["pageSize"], 1);
			/**
			 * set view to create page navigations
			 */
			$this->view->paginate = $paginator->getNavigation();

			/**
			 * set views for search resutls template
			 */
			$searchResultViewObjectsStatus = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);

			//See if Web Service returned error response and redirected to error page
			if($searchResultViewObjectsStatus == WSConstants::$ERROR_RESPONSE){
				return ;
			}

			/**
			 * set the view and move on
			 */
			$this->_request->setControllerName("search")->setActionName("searchresults");
		} else {
			$this->setWSRequestFailed();
		}
	}

	/**
	 * Helper method used to set the search parameters.
	 *
	 * @access private
	 * @return $parameters, containing request information
	 */
	private function setSearchParameters ()
	{
		/**
		 * Set paremeters for request
		 */
		$parameters['pageNo'] = 1;
		$parameters['pageSize'] = $_SESSION["pageSize"];
		return $parameters;
	}

	/**
	 * Helper method used to set search result view objects or data.
	 *
	 * @access private
	 * @param $parameters
	 * @param $phpobject
	 * @param $trail
	 * @return it doesn't return anything
	 */
	private function setSearchResultViewObjects ($parameters, $phpobject, $trail)
	{
		/**
		 * set filters views
		 */
		$this->view->categoryFilter = $phpobject->categoryFilters->cf;
		$this->view->manufacturerFilter = $phpobject->manufacturerFilters->mf;
		$this->view->attributeFilter = $phpobject->attributeFilters->attribute;
		/**
		 * set other results table information in view
		 */
		$outOf = $parameters['pageSize'] * $parameters['pageNo'];
		if ($phpobject['count'] < $outOf)
		$outOf = $phpobject['count'];
		$this->view->numperpage = $outOf;
		$this->view->count = $phpobject['count'];
		$this->view->page = (($parameters['pageNo'] - 1) * $parameters['pageSize']) + 1;
		$this->view->suggestions = $phpobject->keywordPharases->keywordPharase;
		$this->view->pageSize = $_SESSION["pageSize"];
		$categories = $this->getAllCategories();
		if($categories ==WSConstants::$ERROR_RESPONSE){
			return WSConstants::$ERROR_RESPONSE;
		}

		$manufactures = self::getAllManufactures();
		if($manufactures ==WSConstants::$ERROR_RESPONSE){
			return WSConstants::$ERROR_RESPONSE;
		}
		$this->view->displayTrail = $trail->getDisplayTrial($categories, $manufactures);
		/**
		 * set products in view
		 */
		$this->view->products = $this->populateSearchResults($phpobject);
		/**
		 * Render required templates include in search page.
		 */
	    $searchResultsPage = $this->setSearchResultsPage();
        if($searchResultsPage ==WSConstants::$ERROR_RESPONSE){
        	return WSConstants::$ERROR_RESPONSE;
        }		
	}
}
?>