<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once "BaseController.php";
require_once "WSConstants.php";
require_once "Paginator.php";
require_once "SearchTrail.php";
require_once 'InQuireWebUtil.php';
require_once 'ConfigFileManagerModel.php';

/**
 * SearchController is responsible to control all the search actions. However, it is not responsible for filtering
 * actions. 
 *
 * SearchController is used to perform the following actions:
 *
 * <li>Keyword search actions</li>
 * <li>Display category actions</li>
 * <li>Paging actions</li>
 *
 * SearchController also manages the dynamic trail of user search.
 *
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 *
 * @see FilterController
 * @see SearchTrail
 */
class SearchController extends BaseController
{

    /**
     * The default action - show the home page
     */
    public function indexAction ()
    { // TODO Auto-generated SearchController::indexAction() default action
    }

    /**
     * keywordsearchAction is used to perform keyword searches.
     *
     * It initialize a new SearchTrail for every keyword search and gets keywords from GET parameters by using 'keywords' as the parameter name.
     * Newly initiated SearchTrail is then set in the REQUEST as a string with parameter name 'trail'. 'trail' parameter is then used in the view
     * to generate the display search trail.
     *
     * This action uses web service model classes to perform the web services requests. WSFactory is used to create different keyword search requests.
     * WSRequest is used to encapsulate the search keywords. WSRequest instance is than passed to WSGateway which passed send request to spexlive
     * client and finally request is send to http://ws.spexlive.net.
     *
     * WSResponse object is created for the response of http://ws.spexlive.net. WSResponse object contains the XML data returned from hosted
     * web services. This XML data is converted into a php object and then forwarded to view. Successfull response is forwarded to view that generate
     * different filters and search results table with pagination.
     *
     * For any reason web service call is failed (even after retries) than request is forwarded to the ErrorController.
     *
     * Following objects are set for the view:
     *
     * <li>categoryFilter, It is used to create categories filter.</li>
     * <li>manufacturerFilter, It is used to create manufacturers filter.</li>
     * <li>attributeFilter, It is used to create attributes filter.</li>
     * <li>trail, It is used to display search trail.</li>
     * <li>productSummary, It is used to display product summarized information.</li>
     * <li>page, It is used to display page no.</li>
     * <li>count, It is used to display result count.</li>
     *
     *
     * @access public
     * @param keywords, this is a request parameter and is used to get keyword query string
     * @param trail, this is a request parameter and is set to store the search trail string
     * @return it doesn't return anything
     *
     * @see SearchTrail
     * @see WSRequest
     * @see WSFactory
     * @see WSGateway
     * @see WSResponse
     * @see ErrorController
     * @see Paginator
     * @see FilterController
     *
     */
    public function keywordsearchAction ()
    {
        /**
         * Get request paramters for keyword.
         */
        $keyword = trim($_GET['keywords']);
        $keyword = str_replace("%2F", "\\", $keyword);
        $keyword = str_replace('\"', '"', $keyword);
        $keyword = str_replace("\'", "'", $keyword);
        $_SESSION["header"] = $keyword;
        //Set Page size to default value of 10
        $_SESSION["pageSize"] = '10';
        /**
         * Replace every space in keyword with "+" sign
         */
        $keyword = str_replace(" ", "+", $keyword);
        $trail = new SearchTrail("");
        $trail->addKeyword($keyword);
        $_REQUEST["trail"] = $trail->getTrialString();
        /**
         * Prepare paramters for WSRequest
         */
        $parameters = $this->setSearchParameters();
        /**
         * String contains url parameters directly appeded to WSRequest url.
         */
        $parameters['appender'] = $trail->toParameterString("");
        /**
         * Array for extra parameters merge with original WSRequest url.
         */
        $params['suggestKeywordPharases'] = 'true';
        $parameters['plugArray'] = $params;
        Logger::info("keywordsearchAction Calling WS");
        $phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);

        if ($phpobject !== false) {
            /*
             * Check  the returned xml contents for Error message 
			 */
        	if(InQuireWebUtil::isErrorResponse($phpobject)){
        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            	return;
        	}

			/**
             * Check if search results retrun zero product then remove actual search from trail.
             */
            if ($phpobject['count'] == 0) {
                $trail->remove($keyword, "kw");
                $_REQUEST["trail"] = urlencode($trail->getTrialString());
            }
            /**
             * create the pagination table
             */
            $paginator = new Paginator(urlencode($trail->getTrialString()));
            $paginator->load(range(0, $phpobject['count'] - 1), $_SESSION["pageSize"], 1);
            /**
             * set view to create page navigations
             */
            $this->view->paginate = $paginator->getNavigation();
            /**
             * set views for search resutls template
             */
            $searchResultViewObjects = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);
            //check if Web Service Returned error and redirected to Error Page
            if($searchResultViewObjects == WSConstants::$ERROR_RESPONSE){
        		return;
        	}
            /**
             * set the view and move on
             */
            $this->_request->setActionName("searchresults");
            //$this->_request->setControllerName("search")->setActionName("keywordsearch");
        } else {
            $this->setWSRequestFailed();
        }
    }

    /**
     * Pagination is used to organize search results in pages and every page contains set of results to maximize searching speed.
     *
     * It also created the SearchTrail from last search that is category and keyword search and apply pagination on it. for every pagination
     * it gets page no and trail string from GET parameters by using 'page' and 'trail' as the parameters name.
     *
     * Newly initiated SearchTrail is then set in the REQUEST as a string with parameter name 'trail'. 'trail' parameter is then used in the view
     * to generate page wise search results.
     *
     * @access public
     * @param page, this is a request parameter and is used to get page no query string
     * @param trail, this is a request parameter and is set to store the search trail string
     * @return it doesn't return anything
     *
     * @see SearchTrail
     * @see WSRequest
     * @see WSFactory
     * @see WSGateway
     * @see WSResponse
     * @see ErrorController
     * @see Paginator
     * @see FilterController
     *
     */
    public function pageAction ()
    {
        /**
         * Create trial from previous search.
         */
        $trail = new SearchTrail($_GET["trail"]);
        /**
         * Set trail in the request to generate parameter links.
         */
        $_REQUEST["trail"] = $trail->getTrialString();
        /**
         * Set the paramters for web services request.
         */
        $parameters = $this->setSearchParameters();
        /**
         * Set any appended paramter list in the rquest paramters
         */
        $parameters['appender'] = $trail->toParameterString("");
        /**
         * Get url paramters for page.
         */
        if (array_key_exists('page', $_GET)) {
            $parameters['pageNo'] = $_GET['page'];
        } else {
            $parameters['pageNo'] = 1;
        }
        Logger::info("pageAction Calling WS");
        $phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);
        if ($phpobject !== false) {
            /*
             * Check  the returned xml contents for Error message 
			 */
        	if(InQuireWebUtil::isErrorResponse($phpobject)){
        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
        		return;
        	}
        	        	
            $page = $parameters['pageNo'];
            /**
             * create the pagination table
             */
            $paginator = new Paginator(urlencode($trail->getTrialString()));
            $paginator->load(range(0, $phpobject['count'] - 1), $_SESSION["pageSize"], $page);
            /**
             * set view to create page navigations
             */
            $this->view->paginate = $paginator->getNavigation();
            /**
             * Excplicitly set displaytrial to show trail elements on every page.
             */
            $categories = $this->getAllCategories();
            //check if Web Service Returned error and redirected to Error Page
            if($categories == WSConstants::$ERROR_RESPONSE){
        		return;
        	}
            $manufactures = self::getAllManufactures();
            //check if Web Service Returned error and redirected to Error Page
            if($manufactures == WSConstants::$ERROR_RESPONSE){
        		return;
        	}
            $this->view->displayTrail = $trail->getDisplayTrial($categories, $manufactures);
            /**
             * set views for search resutls template
             */
            $searchResultViewObjects = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);
            //check if Web Service Returned error and redirected to Error Page
            if($searchResultViewObjects == WSConstants::$ERROR_RESPONSE){
        		return;
        	}            
			/**
             * set the view and move on, @todo why we have keywordSearch in pageAction?
             */
            $this->_request->setActionName("searchresults");
        } else {
            $this->setWSRequestFailed();
        }
    }

    /**
     * PageSize is used to select the no of results that will be shown on each result page.
     *
     * It also created the SearchTrail from last search that is category and keyword search and apply page size on it. for every Page Size
     * it gets trail string from GET parameters by using 'trail' as the parameters name.
     *
     * Newly initiated SearchTrail is then set in the REQUEST as a string with parameter name 'trail'. 'trail' parameter is then used in the view
     * to generate page wise search results.
     *
     * @access public
     * @param page, this is a request parameter and is used to get page no query string
     * @param trail, this is a request parameter and is set to store the search trail string
     * @return it doesn't return anything
     *
     * @see SearchTrail
     * @see WSRequest
     * @see WSFactory
     * @see WSGateway
     * @see WSResponse
     * @see ErrorController
     * @see Paginator
     * @see FilterController
     *
     */
    public function pagesizeAction ()
    {
        if (array_key_exists('pageSizeCombo', $_GET)) {
            $_SESSION["pageSize"] = $_GET["pageSizeCombo"];
        }
        /**
         * Create trial from previous search.
         */
        $trail = new SearchTrail($_GET["trail"]);
        /**
         * Set trail in the request to generate parameter links.
         */
        $_REQUEST["trail"] = $trail->getTrialString();
        /**
         * Set the paramters for web services request.
         */
        $parameters = $this->setSearchParameters();
        /**
         * Set any appended paramter list in the rquest paramters
         */
        $parameters['appender'] = $trail->toParameterString("");
        Logger::info("pagesizeAction Calling WS");

        $phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);
        if ($phpobject !== false) {
            /*
             * Check  the returned xml contents for Error message 
			 */
        	if(InQuireWebUtil::isErrorResponse($phpobject)){
        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            	return;
        	}
        	

			/**
             * create the pagination table
             */
            $paginator = new Paginator(urlencode($trail->getTrialString()));
            $paginator->load(range(0, $phpobject['count']), $_SESSION["pageSize"], 1);
            /**
             * set view to create page navigations
             */
            $this->view->paginate = $paginator->getNavigation();
            /**
             * Excplicitly set displaytrial to show trail elements on every page.
             */
            $categories = $this->getAllCategories();
            //check if Web Service Returned error and redirected to Error Page
            if($categories == WSConstants::$ERROR_RESPONSE){
        		return;
        	}             
        	
            $manufactures = self::getAllManufactures();

            //check if Web Service Returned error and redirected to Error Page
            if($manufactures == WSConstants::$ERROR_RESPONSE){
        		return;
        	}            
            $this->view->displayTrail = $trail->getDisplayTrial($categories, $manufactures);
            /**
             * set views for search resutls template
             */
            $searchResultViewObjects = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);
            //check if Web Service Returned error and redirected to Error Page
            if($searchResultViewObjects == WSConstants::$ERROR_RESPONSE){
        		return;
        	}            
            
            /**
             * set the view and move on.
             */
            $this->_request->setActionName("searchresults");
        } else {
            $this->setWSRequestFailed();
        }
    }

    /**
     * Display categories are kind of saved searches which has different criterias, such as category, manufacturer, attribute and keywords.
     * Display categories create main navigation search bar for any ecommerce application. These are mainly based on categories.
     *
     * categorysearchAction is used to perform display category searches. It uses DCConfigManagerModel to get the display categories information
     * by using the display category id. Display category id is defined in the urls of display category bar links. This action takes that id and
     * passes that to DCConfigManagerModel to get the child categories for that particular display category.
     *
     * Once it has all the child categories for the display category then it sets them in $parameters and pass those $parameters to $wsRequest
     * to form a web service request. Which is then send to $gateway to perform the web service request.
     *
     * WSResponse object is created for the response of http://ws.spexlive.net. WSResponse object contains the XML data returned from hosted
     * web services. This XML data is converted into a php object and then forwarded to view. Successfull response is forwarded to view that generate
     * different filters and search results table with pagination.
     *
     * For any reason web service call is failed (even after retries) than request is forwarded to the ErrorController.
     *
     * Following objects are set for the view:
     *
     * <li>categoryFilter, It is used to create categories filter.</li>
     * <li>manufacturerFilter, It is used to create manufacturers filter.</li>
     * <li>attributeFilter, It is used to create attributes filter.</li>
     * <li>trail, It is used to display search trail.</li>
     * <li>productSummary, It is used to display product summarized information.</li>
     * <li>page, It is used to display page no.</li>
     * <li>count, It is used to display result count.</li>
     *
     *
     * @access public
     * @param keywords, this is a request parameter and is used to get keyword query string
     * @param trail, this is a request parameter and is set to store the search trail string
     * @return it doesn't return anything
     *
     * @see SearchTrail
     * @see WSRequest
     * @see WSFactory
     * @see WSGateway
     * @see WSResponse
     * @see ErrorController
     * @see Paginator
     * @see FilterController
     * @see DCConfigManagerModel
     *
     */
    public function categorySearchAction ()
    {
        $config = ConfigFileManagerModel::getInstance();
        $categoryType = $config->getValue("category", "inQuire");
    	$newCategory = $_GET['selectedCategoryId']."$".$config->getValue("category", "inQuire");
    	$headerName = urldecode($_GET['headerName']);
        //Set Page size to default value of 10
        $_SESSION["pageSize"] = '10';
        /**
         * Array to store individual category ids. 
         */
        $catIds = array();
        /**
         * Creating empty trail for new dc search.
         */
        $trail = new SearchTrail("");
        //If Request is from Main DC Header
        if (array_key_exists('index', $_GET)) {
            $categoryIndex = $_GET['index'];
            /**
             * We will not add any filter category when display category index will be 0 and will return the result of full catalog. This logic is
             * not good, but right now it works perfectly with the SPR catalog.
             */
            if ($categoryIndex != 0) {
                /**
                 * get the child categories information for the given display category id.
                 */
                if (array_key_exists('catIds', $_GET)) {
                    $catIds = explode(",", $_GET['catIds']);
                }
            }
        } else //If Request is from Category Family List on Top of Product Detail Page
		if (array_key_exists('selectedCategoryId', $_GET)) {
            //get the Id selected by user
            $selectedCategoryId = $_GET['selectedCategoryId'];
            //get All First Level Child of user selected id.
            $childCategories = $this->getChildCategories($selectedCategoryId);
            //Error from Web Seervice and redirected to error page
			if($childCategories == WSConstants::$ERROR_RESPONSE){
					return; 
			}
            $catIds = $childCategories;
            //$catIds = $selectedCategoryId;			
        ///$trail->addParentCategory($selectedCategoryId);
        }
        $_SESSION["header"] = $headerName;
        
        // We only need to set parent category in request product category trail.
        if (array_key_exists('selectedCategoryId', $_GET)) {
        	$trail->addChildCategory($newCategory);
        }
        
        for ($i = 0; $i < sizeof($catIds); $i ++) {
            // Add all mapping categories to SearchTrail.
            //$trail->addCategory($catIds[$i], "");
            if (array_key_exists('selectedCategoryId', $_GET)) {
                ///$trail->addDisplayCategory($catIds[$i]);
                
            	// TODO We only need to set child category in request product category trail when we dont 
            	// have that support from WebService.
            	//$trail->addChildCategory($catIds[$i]);
            } else {
                $trail->addDisplayCategory($catIds[$i].'$'.$categoryType);
            }
        }
        $_REQUEST["trail"] = $trail->getTrialString();
        /**
         * Prepare paramters for web service request.
         */
        $parameters = $this->setSearchParameters();
        /**
         * Appended trail for dc search.
         */
        $parameters['appender'] = $trail->toParameterString("");
        /**
         * Call WebServices Gateway and return WSResponse.
         */
        Logger::info("categorySearchAction Calling WS");

        $phpobject = $this->serviceRequest($parameters, WSConstants::$SEARCH_REQUEST);
        if ($phpobject !== false) {
            /*
             * Check  the returned xml contents for Error message 
			 */
        	if(InQuireWebUtil::isErrorResponse($phpobject)){
        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            	return;
        	}        	

			/**
             * Check if search results retrun zero product then remove actual search from trail.
             */
            if ($phpobject['count'] == 0) {
                for ($i = 0; $i < sizeof($catIds); $i ++) {
                    // Remove all mapping categories from SearchTrail.
                    $trail->remove($catIds[$i], "");
                }
                $_REQUEST["trail"] = urlencode($trail->getTrialString());
            }
            /**
             * create the pagination table
             */
            $paginator = new Paginator(urlencode($trail->getTrialString()));
            $paginator->load(range(0, $phpobject['count'] - 1), $_SESSION["pageSize"], 1);
            /**
             * set view to create page navigations
             */
            $this->view->paginate = $paginator->getNavigation();
            /**
             * set views for search resutls template
             */
            $searchResultViewObjects = $this->setSearchResultViewObjects($parameters, $phpobject, $trail);
            //check if Web Service Returned error and redirected to Error Page
            if($searchResultViewObjects == WSConstants::$ERROR_RESPONSE){
        		return;
        	} 
			/**
             * Show the display trail if category request is for parent category search.
             */
            //if(array_key_exists('selectedCategoryId',$_GET)) {				
            //	$this->view->displayTrail = $trail->getDisplayTrial($phpobject);
            //}
            /**
             * set the view and move on
             */
            $this->_request->setActionName("searchresults");
        } else {
            $this->setWSRequestFailed();
        }
    }

    /**
     * lazyfiltersAction is used to fetch list of filters for a given search frm the target service.
     *
     * It initialize a new SearchTrail for every filter request and get filter type from GET parameters by using 'type' as the parameter name.
     * Newly initiated SearchTrail is then set in the REQUEST as a string with parameter name 'trail'. 'trail' parameter is then used in the view
     * to generate the display search trail.
     *
     * This action uses web service model classes to perform the web services requests. WSFactory is used to create different keyword search requests.
     * WSRequest is used to encapsulate the search keywords. WSRequest instance is than passed to WSGateway which passed send request to spexlive
     * client and finally request is send to http://ws.spexlive.net.
     *
     * WSResponse object is created for the response of http://ws.spexlive.net. WSResponse object contains the XML data returned from hosted
     * web services. This XML data is converted into a php object and then forwarded to view. Successfull response is forwarded to view that generate
     * different filters and search results table with pagination.
     *
     * For any reason web service call is failed (even after retries) than request is forwarded to the ErrorController.
     *
     * Following lists output by the lazyFilter:
     *
     * <li>CategoryFitler List, It is used to create categories list.</li>
     * <li>ManufacturerFilter, It is used to create manufacturers list.</li>
     * <li>AttributeFilter, It is used to create attributes list.</li>
     *
     * This is lazy action so it require ajax to call target service and prepare and format the filter list from
     * response xml data.
     * 
     * It will not send data to the view instead its output the data directly from the controller.
     * 
     * @see SearchTrail
     * @see WSRequest
     * @see WSFactory
     * @see WSGateway
     * @see WSResponse
     * @see ErrorController
     * @see FilterController
     */
    public function lazyfiltersAction ()
    {
        //Its Ajax Request so Set No Renderer
        $this->_helper->viewRenderer->setNoRender();
        /**
         * Set trialString from request.
         */
        $trailString = $_REQUEST["trail"];
        /**
         * Get reuqesy paramter type.
         */
        $type = $_GET["type"];
        /**
         * Create new search trial with previous search.
         */
        $trail = new SearchTrail($trailString);
        /**
         * Prepare paramters for WSRequest
         */
        if ($type == "category")
            $params['categoryFilters'] = - 1; else if ($type == "manufacturer")
            $params['manufacturerFilters'] = - 1; else
            $params['attributeFilters'] = - 1;

        /**
         * Array for extra parameters merge with original WSRequest url.
         */
        $params['method'] = 'search';
        $params['resources'] = 'detail';
        $params['productIdsOnly'] = 'true';
        $parameters['plugArray'] = $params;
        /**
         * String contains url parameters directly appeded to WSRequest url.
         */
        $parameters['appender'] = $trail->toParameterString("");
        /**
         * Call WebServices Gateway and return WSResponse.
         */
        Logger::info("lazyfiltersAction Calling WS");

        $phpobject = $this->serviceRequest($parameters, WSConstants::$AJAX_REQUEST);

        if ($phpobject !== false) {
            /*
             * Check  the returned xml contents for Error message 
			 */
        	if(InQuireWebUtil::isErrorResponse($phpobject)){
        		$this->setWSRequestFailed($phpobject->type,$phpobject->message,"Please contact administrator with above mentioned details");
            	return;
        	}
        	
			/**
             * lazy load and format filters list.
             */
            require_once 'SearchFilterListModel.php';
            $formatter = new SearchFilterListModel($phpobject, $trailString);
            echo $formatter->formatFiltersList($type);
        } else {
            $this->setWSRequestFailed();
        }
    }

    /**
     * Helper method used to set the search parameters.
     *
     * @access private
     * @return $parameters, containing request information
     */
    private function setSearchParameters ()
    {
        /**
         * Set paremeters for request
         */
        $parameters['pageNo'] = 1;
        $parameters['pageSize'] = $_SESSION["pageSize"];
        return $parameters;
    }

    /**
     * Helper method used to set search result view objects or data.
     *
     * @access private
     * @param $parameters
     * @param $phpobject
     * @param $trail
     * @return it doesn't return anything
     */
    private function setSearchResultViewObjects ($parameters, $phpobject, $trail)
    {
        /**
         * set filters views
         */
        $this->view->categoryFilter = $phpobject->categoryFilters->cf;
        $this->view->manufacturerFilter = $phpobject->manufacturerFilters->mf;
        $this->view->attributeFilter = $phpobject->attributeFilters->attribute;
        /**
         * set other results table information in view
         */
        $outOf = $parameters['pageSize'] * $parameters['pageNo'];
        if ($phpobject['count'] < $outOf)
            $outOf = $phpobject['count'];
        $this->view->numperpage = $outOf;
        $this->view->count = $phpobject['count'];
        $this->view->page = (($parameters['pageNo'] - 1) * $parameters['pageSize']) + 1;
        $this->view->suggestions = $phpobject->keywordPhrases->keywordPhrase;
    	$this->view->pageSize = $parameters['pageSize'];
        /**
         * set products in view
         */
        $this->view->products = $this->populateSearchResults($phpobject);
        /**
         * Render required templates include in search page.
         */
        $searchResultsPage = $this->setSearchResultsPage();
        if($searchResultsPage == WSConstants::$ERROR_RESPONSE){
        	return WSConstants::$ERROR_RESPONSE;
        }
    }

    private function preparePaginationDisplayTrail ($phpobject)
    {
    }
}
