<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/


/**
 * SearchTrail class is use to to keep track of filters applied on search results. Its a
 * Wrapper class for criteria that maintains a 'trail' of filters added to the current search. 
 * 
 * <p>
 * It is used on places where previous search needed to start new search or filters search criteria based on 
 * previous search results. All filters applied on single search safe in the object of this class.
 * 
 * <p> 
 * Each criteria is added independent of others, allowing for history, tracking, and subsequently choosing to 
 * backup to a previous result, or modify a single criteria filter.
 * 
 * This class object would traditionally be used in a client environment such as a website showing the "trail" of 
 * filters/criteria applied to achieve the current search results and therefore allow a user to choose to remove 
 * or revert criteria.
 * 
 * Example (4 criteria added each individually):
 * <ul>
 * <li>DisplayCategory: Computer Systems</li>
 * <li>Keyword: ibm</li>
 * <li>Category: Notebook</li>
 * <li>Keyword: thinkpad</li>
 * </ul> 
 * 
 * At each of the 3 above steps the user filtered the results further, and creating a "bread-crumb trail" 
 * for them with 4 items.
 * <p>
 * Once they've narrowed it down, they may choose they want to expand the results by removing the 3rd criteria item.
 * The interface could allow them to do so, and the SearchTrail object told to remove criteria item #3.
 * <p>
 * The resultant criteria would then have:
 * <ul>
 * <li>DisplayCategory: Computer Systems</li>
 * <li>Keyword: ibm</li>
 * <li>Keyword: thinkpad</li>
 * </ul>
 * 
 * In this way, it allows history of what choices a user has made, and lets them see exactly why they are 
 * seeing the results they are, and change them accordingly.* 
 * 
 * Type of Criteria/Filters added in the SearchTrail:
 *
 * <li>Keyords</li>
 * <li>Categories</li>
 * <li>Manufacturers</li>
 * <li>Absolute/Non Absolute Attributes</li>
 *
 * SearchTrail also manages the display list for the trail object.
 * 
 * For complete usage of SearchTrail object
 * 
 * @see SearchController
 * @see FilterController
 * @see WSRequest
 *
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 *
 */
class SearchTrail
{
    /**
     * variable to safe whole trail in string format in the form of query parameters.
     *
     * @var string
     */
    private $searchTrail = "";
    /**
     * Contant Query paramter keys for all criteria/filters support by SearchTrail 
     */
    public static $KEYWORD_FILTER = "&keywordFilter=";
    public static $PARENT_CATEGORY_FILTER_ID = "&parentcategoryFilterId=";
    public static $CHILD_CATEGORY_FILTER_ID = "&childcategoryFilterId=";
    public static $DISPLAY_CATEGORY_FILTER_ID = "&displaycategoryFilterId=";
    public static $CATEGORY_FILTER_ID = "&categoryFilterId=";
    public static $MANUFACTURER_FILTER_ID = "&manufacturerFilterId=";
    public static $ATTRIBUTE_FILTER = "&attributeFilter=";
    public static $ABSOLUTE_ATTRIBUTE_DISPLAY_FILTER = "&absoluteDisplay=";
    /**
     *  Operator contants for absolute attributes.
     */
    public static $EQUALTO = 0;
    public static $GREATERTHAN_OR_EQUAL_TO = 1;
    public static $LESSTHAN_OR_EQUAL_TO = 2;

    /**
     * SearchTrail object contructor.
     * 
     * @param string $searchTrail
     */
    public function __construct ($searchTrail)
    {
        $this->searchTrail = $searchTrail;
    }

    /**
     * This method update the SearchTrail with the keyword passed and fitler the result base on that keyword.
     * It will add keyword along with its required paramter key.
     *
     * @param string $keyword
     */
    public function addKeyword ($keyword)
    {
        $this->searchTrail = $this->searchTrail . "&" . "kw" . "=" . $keyword;
    }

    /**
     * Add display category id in SearchTrail those are used in search. DiplayCategory are just a collection of
     * category ids those are replaced by category ids when trial is append to WebService request. Its only used to
     * distingush between filters those of normal category criteria or display category criteria.
     * 
     * <code>For e.g. dipslaycategoryFilterId=id</code>
     *
     * @param int $id 
     */
    public function addDisplayCategory ($id)
    {
        $this->searchTrail = $this->searchTrail . self::$DISPLAY_CATEGORY_FILTER_ID . $id;
    }

    /**
     * Add category id in the SearchTrail those are used in search. Multiple categores are added in the
     * SearchTrail by adding $CATEGORY_FILTER_ID in AND fashion.	 
     *
     * @param int $id 
     */
    public function addCategory ($id)
    {
        $this->searchTrail = $this->searchTrail . self::$CATEGORY_FILTER_ID . $id;
    }

    /**
     * Add parent category id in SearchTrail those are used in searchParentCategory are just a collection of
     * category ids those are replaced by category ids when trial is append to WebService request. Its only used 
     * to distingush between filters those of normal category criteria or parent category criteria.
     *
     * @param int $id
     * @param string $value
     */
    public function addParentCategory ($id)
    {
        $this->searchTrail = $this->searchTrail . self::$PARENT_CATEGORY_FILTER_ID . $id;
    }

    /**
     * Add child category id in SearchTrail those are used in search. Child Category are just a collection of
     * category ids those are replaced by category ids when trial is append to WebService request. Its only used to
     * distingush between filters those of normal category criteria or child category criteria.
     * 
     * @param int $id
     * @param string $value
     */
    public function addChildCategory ($id)
    {
        $this->searchTrail = $this->searchTrail . self::$CHILD_CATEGORY_FILTER_ID . $id;
    }

    /**
     * Add manufacturer id in SearchTrail those are used in search. Multiple manufactures are added 
     * in the SearchTrail by adding $MANUFACTURER_FILTER_ID in AND fashion.
     *
     * @param int $id
     * @param string $value
     */
    public function addManufacturer ($id)
    {
        $this->searchTrail = $this->searchTrail . self::$MANUFACTURER_FILTER_ID . $id;
    }

    /**
     * Add attribute id and value in SearchTrail those are use in search. Multiple attributes are added in the
     * SearchTrail by adding $ATTRIBUTE_FILTER in AND fashion.
     *
     * @param int $id
     * @param string $value
     */
    public function addAttribute ($id, $value)
    {
        $this->attributeId = $id;
        $this->attributeValue = $value;
        $attr = $id . "+" . $value;
        $this->searchTrail = $this->searchTrail . self::$ATTRIBUTE_FILTER . $attr;
    }

    /**
     * Add absolute attribute in SearchTrail. Absolute attributes computes by there value and operator.
     * multiple absolute attribtues are added in AND fashion.
     *
     * @param int $id
     * @param float $value
     * @param string $displayValue
     * @param string $operator
     */
    public function addAbsoluteAttribute ($id, $value, $operator)
    {
        $this->attributeId = $id;
        $this->attributeValue = $value;
        if ($operator == self::$EQUALTO)
            $attr = $id . "==" . $value;
        else if ($operator == self::$GREATERTHAN_OR_EQUAL_TO)
            $attr = $id . ">=" . $value;
        else
            $attr = $id . "<=" . $value;
        $this->searchTrail = $this->searchTrail . self::$ATTRIBUTE_FILTER . $attr;
        /*
		 *  $ABSOLUTE_ATTRIBUTE_DISPLAY_FILTER is only used to display selected filter value in display trail.
		 */
        //$this->searchTrail = $this->searchTrail . self::$ABSOLUTE_ATTRIBUTE_DISPLAY_FILTER . $displayValue;
    }

    /**
     * Remove given filter type from the SearchTrail.
     *
     * @param int $id
     * @param string filter $type
     */
    public function remove ($id, $type)
    {
        $newTrail = '';
        $attr = explode("&", $this->getTrialString());
        for ($i = 1; $i < sizeof($attr); $i ++) {
            if ($attr[$i] == '')
                continue;
            $filters = explode("=", $attr[$i]);
            $filter = $filters[0];
            if ($filter == $type) {
                $attrId = - 1;
                if ($type != "kw") {
                    $attribute = explode("+", $filters[1]);
                    $attrId = $attribute[0];
                } else {
                    $attrId = $filters[1];
                }
                if ($id != $attrId) {
                    $newTrail = $newTrail . '&' . $attr[$i];
                }
            } else {
                $newTrail = $newTrail . '&' . $attr[$i];
            }
        }
        $this->searchTrail = $newTrail;
    }

    /**
     * Clear the SearchTrail. All the filter added in the trail are clear and resets the whole search.
     *
     */
    public function clear ()
    {
        $this->searchTrail = "";
    }

    /**
     * Check that the given trail contains any category filter id. Returns true if the trail contains category filter id
     * else return false.
     *
     * @param string $trailString
     * @return boolean
     */
    public function isCategoryFilterExists ($trailString)
    {
        $params = explode("&", $trailString);
        for ($i = 0; $i < sizeof($params); $i ++) {
            $filter = explode("=", $params[$i]);
            if ($filter[0] == "categoryFilterId")
                return true;
        }
        return false;
    }

    /**
     * Return string contaning paramters that are use in filtering requests. This method remove all parameters
     * those are used only to maintains trail filters and distinguish between there types. The results trail that
     * used in Web Service request not requires those parameters.
     *
     * The Parameter action used to check that the request comming from client is from category filter. 
     * 
     * @param string $action
     * @return string parameters_url
     */
    public function toParameterString ($action)
    {
        $trail = "";
        /**
         * Prepare paramter string for keywordFilter.
         */
        $trail = $this->prepareKeywordFilter();
        /**
         * Append remaining trail into trail.
         */
        $trail .= $this->prepareTrailWithoutDisplayParameters();
        /**
         * Remove special charaters
         */
        $trail = $this->removeSpecialCharacters($trail);
        if ($action != "category" and ! $this->isCategoryFilterExists($trail)) {
            $trail = $this->prepareDisplayCategoryFilter($trail);
            $trail = $this->prepareChildCategoryFilter($trail);
        } else {
            $trail = $this->prepareParentCategoryFilter($trail);
        }
        return $trail;
    }

    /**
     * Return url string representation of trail that is use in filtering requests. Its will return all the paramters
     * present in trail irrespective of their types those are meaningful for web service request.
     *
     * @return string parameteric url
     */
    public function getTrialString ()
    {
        return $this->searchTrail;
    }

    /**
     * This method will collect all keywords from the trail and prepare them to pass to web service request.
     * The SearchTrail holds keyword in kw parameter that will convert to actuall keyworFilter parameter before
     * sending the paramter string to web service request.
     * 
     * @see SearchTrail->toParameterString.
     *
     * @return string
     */
    public function prepareKeywordFilter ()
    {
        $keyword = '';
        $attr = explode("&", $this->searchTrail);
        for ($i = 0; $i < sizeof($attr); $i ++) {
            $filters = explode("=", $attr[$i]);
            if ($filters[0] == 'kw') {
                $keyword .= $filters[1];
                $keyword .= "+";
            }
        }
        if ($keyword == '')
            return "";
        $newKeywordTrail = self::$KEYWORD_FILTER . substr($keyword, 0, strlen($keyword) - 1);
        $newKeywordTrail = str_replace(' ', '+', $newKeywordTrail);
        return $newKeywordTrail;
    }

    /**
     * This method will remove any special characters in the SearchTrail generated trialString. Current URLDecoding
     * unable to decode there special charaters.
     *
     * @param string $trail
     * @return decoded_trail
     */
    public function removeSpecialCharacters ($trail)
    {
        $trail = str_replace('\"', '"', $trail);
        $trail = str_replace("\'", "'", $trail);
        $trail = str_replace('\\', '', $trail);
        $trail = str_replace('\\', '', $trail);
        return $trail;
    }

    /**
     * This method is used to remove all paramters those have only used to display there values not used by
     * web service at all. 
     *
     * @return string
     */
    public function prepareTrailWithoutDisplayParameters ()
    {
        $newTrail = '';
        $parameters = explode("&", $this->getTrialString());
        for ($i = 1; $i < sizeof($parameters); $i ++) {
            if ($parameters[$i] == '')
                continue;
            $filters = explode("=", $parameters[$i]);
            $filter = $filters[0];
            if ($filter != 'absoluteDisplay') {
	            if ($filter == 'attributeFilter') {
	            	if ($this->containsOperator($filters[1]) or $filters[3]) {
	            		$temp = explode("!", $parameters[$i]);
	            		$newTrail = $newTrail . '&' . $temp[0];
	            	} else {
		                $attribute = explode("+", $filters[1]); 
	                    $id = $attribute[0];
	                    $temp = explode("!", $attribute[1]);
	                    $value = $temp[0];
	                    /*
	                    for ($j = 1; $j < sizeof($attribute); $j ++) {
	                        $value = $value . ' ' . $attribute[$j];
	                    }
	                    */
	                    $value = trim($value);
						$value = utf8_decode($value);
						$value = str_replace(" ", "+", $value);
						$attrData = $id . '+' . $value;
	                    $newTrail = $newTrail . self::$ATTRIBUTE_FILTER . $attrData;	                    
	            	}	                
	            } else
	                $newTrail = $newTrail . '&' . $parameters[$i];
            }
        }
        return $newTrail;
    }

    public function prepareTrailWithoutDisplayParameters1 ()
    {
        /**
         * 
         * Remove every parameters from SearchTrail becuase request dont need display parameter.
         * It just present in SearchTrail for showing its DisplayValue in DisplayTrail.
         * 
         */
        $newTrail = '';
        $parameters = explode("&", $this->getTrialString());
        for ($i = 1; $i < sizeof($parameters); $i ++) {
            if ($parameters[$i] == '')
                continue;
            $filters = explode("=", $parameters[$i]);
            $filter = $filters[0];
            if ($filter != 'absoluteDisplay') {
                $newTrail = $newTrail . '&' . $parameters[$i];
            }
        }
        return $newTrail;
    }

    /**
     * This method prepare the trail for web service request and replace all display category ids with category ids.
     *
     * @param string $trail
     * @return string
     */
    public function prepareDisplayCategoryFilter ($trail)
    {
        /**
         * Replace "child" word from childCategoryId paramters becuase that's only use to distinguish them
         * in DisplayTrail and Trail category values.
         */
        $trailString = str_replace("display", "", $trail);
        return $trailString;
    }

    /**
     * This method prepare the trail for web service request and replace all child category ids with category ids.
     *
     * @param string $trail
     * @return string
     */
    public function prepareChildCategoryFilter ($trail)
    {
        /**
         * Replace "display" word becuase that's only to restrict DisplayCategory to 
         * display in trial 
         */
        $trailString = str_replace("child", "", $trail);
        return $trailString;
    }

    /**
     * This method prepare the trail for web service request and replace all parent category ids with category ids.
     *
     * @param string $trail
     * @return string
     */
    public function prepareParentCategoryFilter ($trail)
    {
        /**
         * Replace "display" word becuase that's only to restrict DisplayCategory to 
         * display in trial 
         */
        $trailString = str_replace("parent", "", $trail);
        return $trailString;
    }

    /**
     * Return List of Filters/Criteria available in SearchTrail to display.
     *
     * @return array
     */
    public function getDisplayTrial ($categories, $manufactures)
    {
        $attr = explode("&", $this->getTrialString());
        $trailList = false;
        $keywordSearch = false;
        $id = "";
        $value = "";
        for ($i = 1; $i < sizeof($attr); $i ++) {
            if ($attr[$i] == '')
                continue;
            $filters = explode("=", $attr[$i]);
            $filter = $filters[0];
            //if (!$this->contains($filter)) continue;
            if ($filter == 'attributeFilter') {
                $operator = $this->getOperator($filters[1]);
                if ($this->containsOperator($filters[1]) or $filters[3]) {
                	$attribute = explode($operator, $filters[1]);
                	$temp = explode('!', $attr[$i]);
                	$id = $attribute[0];
                    $displayVal = $temp[1];
                	//$absDisplayAttr = explode("=", $attr[$i + 1]);
                    $displayVal = str_replace('\"', '"', $displayVal);
                    $displayVal = str_replace('\\', '', $displayVal);
                    
                    $value = $operator . ' ' . $displayVal;
                    $element['filter'] = $filter; // . '|absoluteDisplay';
                    $element['id'] = $id;
                    $element['value'] = $value;
                    $element['display'] = $value;
                    $trailList[$i] = $element;
                } else {
                    $attribute = explode("+", $filters[1]);
                    $id = $attribute[0];
                    $temp = explode("!", $attribute[1]);
                    $value = $temp[0];
                    /*
                    for ($j = 1; $j < sizeof($attribute); $j ++) {
                        $value = $value . ' ' . $attribute[$j];
                    }
                    */
                    $value = str_replace('%2B', '+', $value);
                    $value = str_replace("\'", "'", $value);
                    $value = str_replace('\\', '', $value);
                    $element['filter'] = $filter;
                    $element['id'] = $id;
                    $element['value'] = $value;
                    $element['display'] = $temp[1];
                    $trailList[$i] = $element;
                }
            } else if ($filter == 'manufacturerFilterId') {
                $id = $filters[1];
                $element['filter'] = $filter; // . '|manufacturerFilterValue';
                $element['id'] = $id;
                /**
                 * Retreiving manufacturer name
                 */
                $mfgName = "";
                foreach ($manufactures as $key => $value) {
                    if ($key == $id)
                        $mfgName = $value;
                }
                $element['value'] = $mfgName;
                $element['display'] = $mfgName;
                $trailList[$i] = $element;
            } else if ($filter == 'categoryFilterId') {
                $id = $filters[1];
                $element['filter'] = $filter; // . '|categoryFilterValue';
                $element['id'] = $id;
                /**
                 * Retreiving category name
                 */
                $catName = "";
                foreach ($categories as $key => $value) {
                    if ($key == $id)
                        $catName = $value;
                }
                $element['value'] = $catName;
                $element['display'] = $catName;
                $trailList[$i] = $element;
            } else if ($filter == 'parentcategoryFilterId') {
                $element['filter'] = $filter;
                $id = $filters[1];
                //For parent category id get all child categories from ajax request.
                $parameters = explode("&", $this->getTrialString());
                $childIds = "";
                for ($j = 1; $j < sizeof($parameters); $j ++) {
                    $values = explode("=", $parameters[$j]);
                    $parameter = $values[0];
                    if ($parameter == 'childcategoryFilterId') {
                        $childIds .= "|" . $values[1];
                    }
                }
                $id .= $childIds;
                // ********************************************
                $element['id'] = $id;
                $value = "";
                if (count($phpobject->categoryFilters) > 0) {
                    foreach ($phpobject->categoryFilters->cf as $category) {
                        if ($category["id"] == $filters[1])
                            $value = $category["name"];
                    }
                }
                $element['value'] = "ABC " . $value;
                $element['display'] = "ABC " . $value;
                $trailList[$i] = $element;
            } else if ($filter == 'keywordFilter') {
                $keywords = explode("+", $filters[1]);
                $keywordsList = false;
                for ($j = 0; $j < sizeof($keywords); $j ++) {
                    $element['filter'] = $filter;
                    $element['id'] = $keywords[$j];
                    $element['value'] = $keywords[$j];
                    $element['display'] = $keywords[$j];
                    $keywordsList[$j] = $element;
                }
                $trailList[$i] = $keywordsList;
            } else if ($filter == 'kw') {
                if ($i == 1)
                    continue;
                $keywordSearch = true;
                $id = $filters[1];
                $id = $this->removeSpecialCharacters($id);
                $value = str_replace("+", " ", $filters[1]);
                $value = $this->removeSpecialCharacters($value);
                $element['filter'] = $filter;
                $element['id'] = $filters[1];
                $element['value'] = $value; //str_replace("+", " ", $filters[1]);
                $element['display'] = $value;
                $trailList[$i] = $element;
            }
        }
        return $trailList;
    }

    /**
     * Compress Array used to remove empty values from array.
     *
     * @param unknown_type $compress
     * @return unknown
     */
    private function compressArray ($compress)
    {
        $newArray = null;
        $counter = 0;
        for ($i = 0; $i < sizeof($compress); $i ++) {
            if ($compress[$i] != '') {
                $newArray[$counter ++] = $compress[$counter ++];
            }
        }
        return $newArray;
    }

    /**
     * Check that supplied keyword is available in the particular array.
     *
     * @param string $keyword
     * @return boolean
     */
    public function contains ($keyword)
    {
        $filters = array('kw' , 'categoryFilterId' , 'displaycategoryFilterId' , 'manufacturerFilterId' , 'attributeFilter' , '');
        for ($i = 0; $i < sizeof($filters); $i ++) {
            if (trim($keyword) == $filters[$i]) {
                return true;
            }
        }
        return false;
    }

    /**
     * Check that operator is avalable in the gives value.
     *
     * @param string $value
     * @return boolean
     */
    public function containsOperator ($value)
    {
        $operators = array('=' , '>' , '<');
        for ($i = 0; $i < sizeof($operators); $i ++) {
            if (strpos($value, $operators[$i]) > 0)
                return true;
        }
        return false;
    }

    /**
     * Return the specific operator present in the given value.
     *
     * @param string $value
     * @return string operator
     */
    public function getOperator ($value)
    {
        $operators = array('=' , '>' , '<');
        for ($i = 0; $i < sizeof($operators); $i ++) {
            if (strpos($value, $operators[$i]) > 0) {
                return $operators[$i] . "=";
            }
        }
        return '==';
    }
}
?>