<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html
*/

require_once 'ConfigFileManagerModel.php';
require_once 'WSException.php';
require_once 'Logger.php';

/**
 * WSRequest class is used to send web services request to WSGateway class. This class i sa wrapper around REST based
 * requests. It contains methods that is need by the target service and some utility methods.
 * 
 * WSRequest class is used to create the following requests:
 *
 * <li>Search Request (Keyword and Dicplay Categories Search)</li>
 * <li>Product Details Request</li>
 * <li>Compare Products Request</li>
 * <li>Category Family Request</li>
 * <li>Ajax Request for Lazy Details</li>
 * 
 * WSRequest also provide information about the target service , the service endpoints, method used and the
 * list of paramters required by the request. <br>
 * 
 * This class throws WSException when created WSrequest is not validate.
 * 
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 * 
 * @see WSGateway
 * @see WSException
 *
 */
class WSRequest
{
    /**
     * request parameter
     *
     * @var array
     */
    private $parameters;
    /**
     * EndPoint for the REST Service client
     */
    private $endPoint;
    /**
     * Web Service Site ID
     *
     * @var string
     */
    private $siteId;
    /**
     * Associate class for ConfigFileManagerModel
     */
    private $config;

    /**
     * constructor
     * private so only getInstance() method can instantiate object for WSRequest
     * @return void
     */
    private function __construct ()
    {
        $this->initialize();
    }

    /**
     * Initialize WSRequest object and get values for WebServices Endpoints and siteId from configuration file.
     */
    private function initialize ()
    {
        $this->config = ConfigFileManagerModel::getInstance();
        $this->endPoint = $this->getEndPoint();
        $this->siteId = $this->getSiteId();
    }

    /**
     * This method use to validate the WSRequest when creating any request. It returns false if the created request
     * is not correct and will not successfully processed by target service.
     *
     * @param WSRequest $wsRequest
     * @return boolean
     */
    private function validate (WSRequest $wsRequest)
    {
        // TODO List
        // not know right now how exactly validate WSRequests.
        return true;
    }

    /**
     * Return the target service endpoint.
     * 
     * @return string
     */
    public function getEndPoint ()
    {
        return $this->config->getValue('endPoint', 'SLService');
    }

    /**
     * Return Site ID
     *
     * @return string
     */
    public function getSiteId ()
    {
        return $this->config->getValue('siteId', 'SLService');
    }

    /**
     * Get default parameters array of WSRequest object
     *
     * @return array
     */
    public function getParameters ()
    {
        return $this->parameters;
    }

    /**
     * Use to prepare paramter string from paramters array.
     *
     * @return string
     */
    public function getParametersString ()
    {
        $params = $this->getParameters();
        $buffer = "";
        foreach ($params as $key => $value) {
            //if multiple params with same name are passed in the form of array
            if (is_array($value)) {
                foreach ($value as $innerkey => $innervalue) {
                    $buffer .= ($key . '=' . $innervalue . '&');
                }
            } else {
                if ($key == 'appender')
                    continue;
                $buffer .= ($key . '=' . $value . '&');
            }
            //$buffer = substr($buffer, 0, strlen($buffer) - 1);
        //$buffer = substr($buffer, 0, strlen($buffer) );
        }
        //Remove last & sign from the buffer and append 'appender param' if exist
        if (isset($params['appender'])) {
            return (substr($buffer, 0, strlen($buffer) - 1)) . $params['appender'];
        } else {
            return (substr($buffer, 0, strlen($buffer) - 1));
        }
    }

    /**
     * This method will merge default options with user specifies options before
     * sending request to the web service.
     *
     * @param  array  $defaultOptions Default Options
     * @return array Merged array of user and default/required options
     */
    private function _prepareOptions (array $defaultOptions = null, array $options)
    {
        if (isset($defaultOptions)) {
            return array_merge($defaultOptions, $options);
        } else {
            return $options;
        }
    }

    /**
     * This method is used to append paramters passed from outside WSRequest from any source to destination
     * parameters array.
     *
     * @param array $source
     * @param array $destination
     * @return array
     */
    public function appendUrlParameters (array $source, array $destination)
    {
    	if (isset($source['appender']))
			$destination['appender'] = $source['appender'];
    	return $destination;
    }

    /**
     * This method will create keyword search and display category search request depending on the paramters
     * pass to it. This method will return WSException::$REQUEST_VALIDATION_ERROR if the created WSRequest
     * is not valdated properly.
     * 
     * @throws WSException 
     * 
     * @param array $parameters
     * @return WSRequest
     */
    public function createSearchRequest ($parameters)
    {
        $wsRequest = new WSRequest();
        /**
         * Assign some url paramters those are common is all search requests.
         */
        $params['method'] = 'search';
        $params['siteId'] = $wsRequest->getSiteId();
        $params['catalog'] = $wsRequest->config->getValue('catalog', 'inQuire');
        $params['locale'] = $wsRequest->config->getValue('locale', 'inQuire');
        $params['subcatalogFilter'] = $wsRequest->config->getValue('subCatalog', 'inQuire');
        $params['category'] = $wsRequest->config->getValue('category', 'inQuire');
        $params['resources'] = 'detail';
        $params['includeLocales'] = 'false';
        $params['manufacturer'] = 'default';
        
        $skuTypesTemp = explode(",", $wsRequest->config->getValue('skuType', 'SpeX-Plus Add-Ons'));
        $skuTypes = array();

        foreach($skuTypesTemp as $sku) {
        	$skuTypes[] = str_replace(' ', '+', $sku);
        }
        
        $params['skuType'] = $skuTypes;
		
        /**
         * Check that the required paramters exists in the array.
         */
        if (array_key_exists('pageNo', $parameters)) {
            $params['pageNo'] = $parameters['pageNo'];
        }
        if (array_key_exists('pageSize', $parameters)) {
            $params['pageSize'] = $parameters['pageSize'];
        }
        if (array_key_exists('productIdsOnly', $parameters)) {
            $params['productIdsOnly'] = $parameters['productIdsOnly'];
        } else {
            $params['productIdsOnly'] = 'false';
        }
        if (array_key_exists('descriptionTypes', $parameters)) {
            $params['descriptionTypes'] = $parameters['descriptionTypes'];
        } else {
            $params['descriptionTypes'] = '1,2,3';
        }
        if (array_key_exists('categoryFilters', $parameters)) {
            $params['categoryFilters'] = $parameters['categoryFilters'];
        } else {
            $params['categoryFilters'] = 6;
        }
        if (array_key_exists('attributeFilters', $parameters)) {
            $params['attributeFilters'] = $parameters['attributeFilters'];
        } else {
            $params['attributeFilters'] = -1;
        }
        if (array_key_exists('manufacturerFilters', $parameters)) {
            $params['manufacturerFilters'] = $parameters['manufacturerFilters'];
        } else {
            $params['manufacturerFilters'] = 6;
        }
        if (array_key_exists('keywordFilter', $parameters)) {
            $params['keywordFilter'] = $parameters['keywordFilter'];
        }
        /**
         * Appended Paramters is url string containing parameter that is appended with the Request url and pass to
         * target WebService.
         */
        $params = $wsRequest->appendUrlParameters($parameters, $params);
        /**
         * key <b>plugArray</b> contains an array of paramters those are merged with and default paramter and
         * pass to the WebService.
         */
        if (array_key_exists('plugArray', $parameters)) {
            $wsRequest->parameters = $wsRequest->_prepareOptions($parameters['plugArray'], $params);
        } else {
            $wsRequest->parameters = $params;
        }
        /**
         * Validate and created WSRequest if true it will return WSrequest to call WebService else return
         * Exception code <b>REQUEST_VALIDATION_ERROR</b>
         */
        if (! self::validate($wsRequest)) {
            throw new WSException("[WSException] ==> Invalid Request.", WSException::$REQUEST_VALIDATION_ERROR);
            return WSException::$REQUEST_VALDATION_ERROR;
        }
        return $wsRequest;
    }

    /**
     * This method will create product details sheet request. This method will return 
     * WSException::$REQUEST_VALIDATION_ERROR if the created WSRequest is not valdated properly.
     * 
     * @throws WSException
     *
     * @param array $parameters
     * @return WSRequest
     */
    public function createProductDetailRequest ($parameters)
    {
        $wsRequest = new WSRequest();
        /**
         * Assign some url paramters those are common is all product requests.
         */
        $parameters['method'] = 'getProduct';
        $parameters['siteId'] = $wsRequest->getSiteId();
        $parameters['catalog'] = $wsRequest->config->getValue('catalog', 'inQuire');
        $parameters['locale'] = $wsRequest->config->getValue('locale', 'inQuire');
        $parameters['subcatalogFilter'] = $wsRequest->config->getValue('subCatalog', 'inQuire');
        $parameters['category'] = $wsRequest->config->getValue('category', 'inQuire');
        $parameters['accessoryMax'] = $wsRequest->config->getValue('accessories', 'SpeX-Plus Add-Ons');
        $parameters['upsellMax'] = $wsRequest->config->getValue('upsell', 'SpeX-Plus Add-Ons');
        $parameters['similarMax'] = $wsRequest->config->getValue('similar', 'SpeX-Plus Add-Ons');
        
	    $skuTypesTemp = explode(",", $wsRequest->config->getValue('skuType', 'SpeX-Plus Add-Ons'));
        $skuTypes = array();

        foreach($skuTypesTemp as $sku) {
        	$skuTypes[] = str_replace(' ', '+', $sku);
        }
        
        $params['skuType'] = $skuTypes;
        
        $wsRequest->parameters = $parameters;
        /**
         * Validate and created WSRequest if true it will return WSrequest to call WebService else return
         * Exception code <b>REQUEST_VALIDATION_ERROR</b>
         */
        if (! self::validate($wsRequest)) {
            throw new WSException("[WSException] ==> Invalid Request.", WSException::$REQUEST_VALIDATION_ERROR);
            return WSException::$REQUEST_VALDATION_ERROR;
        }
        return $wsRequest;
    }

    /**
     * This method will create request to fetch all categories , there parent and child categories tree.
     * This method will return WSException::$REQUEST_VALIDATION_ERROR if the created WSRequest is not 
     * valdated properly.
     * 
     * @throws WSException
     *
     * @param array $parameters
     * @return WSRequest
     */
    public function createCategoryFamilyRequest ($parameters)
    {
        $wsRequest = new WSRequest();
        /**
         * Assign some url paramters those are common is all product requests.
         */
        $parameters['method'] = 'getCategoryTree';
        $parameters['catalog'] = $wsRequest->config->getValue('catalog', 'inQuire');
        $parameters['locale'] = $wsRequest->config->getValue('locale', 'inQuire');
        $parameters['siteId'] = $wsRequest->getSiteId();
        $parameters['childLevels'] = '-1';
        $parameters['type'] = $wsRequest->config->getValue('category', 'inQuire');
        
        $wsRequest->parameters = $parameters;
        /**
         * Validate and created WSRequest if true it will return WSrequest to call WebService else return
         * Exception code <b>REQUEST_VALIDATION_ERROR</b>
         */
        if (! self::validate($wsRequest)) {
            throw new WSException("[WSException] ==> Invalid Request.", WSException::$REQUEST_VALIDATION_ERROR);
            return WSException::$REQUEST_VALDATION_ERROR;
        }
        return $wsRequest;
    }

    /**
     * 
     * This method will create product comparison request. This method will return 
     * WSException::$REQUEST_VALIDATION_ERROR if the created WSRequest is not valdated properly.
     * 
     * @see WSException
     *
     * @param array $parameters
     * @return WSRequest
     */
    public function createCompareProductRequest ($parameters)
    {
        $wsRequest = new WSRequest();
        /**
         * Assign some url paramters those are common is all product requests.
         */
        $params['method'] = 'compareProducts';
        $params['siteId'] = $wsRequest->getSiteId();
        $params['catalog'] = $wsRequest->config->getValue('catalog', 'inQuire');
        $params['locale'] = $wsRequest->config->getValue('locale', 'inQuire');
        $params['subcatalogFilter'] = $wsRequest->config->getValue('subCatalog', 'inQuire');
        $params['category'] = $wsRequest->config->getValue('category', 'inQuire');
        $params['productIds'] = $parameters['productIds'];
        if (array_key_exists('includeSkus', $params)) {
            $params['includeSkus'] = 'true';
        }
        $params['descriptionTypes'] = 'all';
        $params['resourceType'] = 'detail';
        $params = $wsRequest->appendUrlParameters($parameters, $params);
        if (array_key_exists('plugArray', $parameters)) {
            $wsRequest->parameters = $wsRequest->_prepareOptions($parameters['plugArray'], $params);
            //$params;
        } else {
            $wsRequest->parameters = $params;
        }
        /**
         * Validate and created WSRequest if true it will return WSrequest to call WebService else return
         * Exception code <b>REQUEST_VALIDATION_ERROR</b>
         */
        if (! self::validate($wsRequest)) {
            throw new WSException("[WSException] ==> Invalid Request.", WSException::$REQUEST_VALIDATION_ERROR);
            return WSException::$REQUEST_VALDATION_ERROR;
        }
        return $wsRequest;
    }

    /**
     * 
     * This method will create request to get list of categories of entire catalog. This method will return 
     * WSException::$REQUEST_VALIDATION_ERROR if the created WSRequest is not valdated properly.
     * 
     * @see WSException
     *
     * @param array $parameters
     * @return WSRequest
     */
    public function createCategoryDirectoryRequest ($parameters)
    {
        $wsRequest = new WSRequest();
        /**
         * Assign some url paramters those are common is all product requests.
         */
        $params['method'] = 'getCategories';
        $params['siteId'] = $wsRequest->getSiteId();
        $params['catalog'] = $wsRequest->config->getValue('catalog', 'inQuire');
        $params['locale'] = $wsRequest->config->getValue('locale', 'inQuire');
        $params['subcatalogFilter'] = $wsRequest->config->getValue('subCatalog', 'inQuire');
        $params['type'] = $wsRequest->config->getValue('category', 'inQuire');
        $params = $wsRequest->appendUrlParameters($parameters, $params);
        if (array_key_exists('plugArray', $parameters)) {
            $wsRequest->parameters = $wsRequest->_prepareOptions($parameters['plugArray'], $params);
        } else {
            $wsRequest->parameters = $params;
        }
        /**
         * Validate and created WSRequest if true it will return WSrequest to call WebService else return
         * Exception code <b>REQUEST_VALIDATION_ERROR</b>
         */
        if (! self::validate($wsRequest)) {
            throw new WSException("[WSException] ==> Invalid Request.", WSException::$REQUEST_VALIDATION_ERROR);
            return WSException::$REQUEST_VALDATION_ERROR;
        }
        return $wsRequest;
    }

    /**
     * 
     * This method will create request to get list of manufacturers of entire catalog. This method will return 
     * WSException::$REQUEST_VALIDATION_ERROR if the created WSRequest is not valdated properly.
     * 
     * @see WSException
     *
     * @param array $parameters
     * @return WSRequest
     */
    public function createManufacturerDirectoryRequest ($parameters)
    {
        $wsRequest = new WSRequest();
        /**
         * Assign some url paramters those are common is all product requests.
         */
        $params['method'] = 'getManufacturers';
        $params['siteId'] = $wsRequest->getSiteId();
        $params['catalog'] = $wsRequest->config->getValue('catalog', 'inQuire');
        $params['locale'] = $wsRequest->config->getValue('locale', 'inQuire');
        $params['subcatalogFilter'] = $wsRequest->config->getValue('subCatalog', 'inQuire');
        $params = $wsRequest->appendUrlParameters($parameters, $params);
        if (array_key_exists('plugArray', $parameters)) {
            $wsRequest->parameters = $wsRequest->_prepareOptions($parameters['plugArray'], $params);
            //$params;
        } else {
            $wsRequest->parameters = $params;
        }
        /**
         * Validate and created WSRequest if true it will return WSrequest to call WebService else return
         * Exception code <b>REQUEST_VALIDATION_ERROR</b>
         */
        if (! self::validate($wsRequest)) {
            throw new WSException("[WSException] ==> Invalid Request.", WSException::$REQUEST_VALIDATION_ERROR);
            return WSException::$REQUEST_VALDATION_ERROR;
        }
        return $wsRequest;
    }

    /**
     * This methid is used to set every ajax call to web service. Parameters are passed to make it flexible
     * of every ajax call.
     *
     * @param array $parameters
     * @return WSRequest
     */
    public function createWSAjaxRequest ($parameters)
    {
        $wsRequest = new WSRequest();
        $params['siteId'] = $wsRequest->getSiteId();
        $params = $wsRequest->appendUrlParameters($parameters, $params);
        if (array_key_exists('plugArray', $parameters))
            $wsRequest->parameters = $wsRequest->_prepareOptions($parameters['plugArray'], $params); else
            $wsRequest->parameters = $params;
        /**
         * Validate and created WSRequest if true it will return WSrequest to call WebService else return
         * Exception code <b>REQUEST_VALIDATION_ERROR</b>
         */
        if (! self::validate($wsRequest)) {
            throw new WSException("[WSException] ==> Invalid Request.", WSException::$REQUEST_VALIDATION_ERROR);
            return WSException::$REQUEST_VALDATION_ERROR;
        }
        return $wsRequest;
    }

    /**
     * Enter description here...
     *
     * @return unknown
     */
    // TODO createRequestString and getRequestURL are for same purpose use only one here.	 
    public function createRequestString ()
    {
        $requestURL = $this->getEndPoint() . '&' . $this->getParametersString();
        //print_r($requestURL);
        return $requestURL; 
    }

    /**
     * Return request url
     *
     * @return string
     */
    public function getRequestURL ()
    {
        $requestURL = $this->getEndPoint() . '&' . $this->getParametersString();
        //print_r($requestURL);
        return $requestURL;
    }

    /**
     * Return string containing the parameters passed to the Web Service.
     *
     * @return unknown
     */
    public function getQueryString ()
    {
    	//print_r($this->getParametersString());
        return $this->getParametersString();
    }
}
?>