<?php

/**
* inQuirePHP
*
* LICENSE
*
* This source file is subject to Apache 2.0 LICENSE that is bundled
* with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://www.apache.org/licenses/LICENSE-2.0.html
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to integrate@etilize.com so we can send you a copy immediately.
*
* @copyright  Copyright (c) 2007-2008 Etilize USA Inc. (http://www.etilize.com)
* @license    http://www.apache.org/licenses/LICENSE-2.0.html 
* 
*/

require_once 'WSException.php';
require_once 'InQuireWebUtil.php';

/**
 * WSResponse class represents the web services response. It will create different types of response required
 * by the client.
 * 
 * WSResponse contains XML data retreive from target service. It contains some utility method to process response data.
 *
 * WSResponse also provide information about the status of the response and the response message if WebService
 * is failed.
 * 
 * This class throws WSException when created WSResponse is not validate.
 * 
 * @author Saad Rashid (msrashid@etilizepak.com)
 * @version 1.0
 * 
 * @see WSGateway
 * @see WSException
 *
 */
class WSResponse
{
    private $response = null;

    /**
     * constructor
     * private so only getInstance() method can instantiate
     * @return void
     */
    private function __construct ($response)
    {
        $this->response = $response;
    }

    /**
     * Create Zend Http WSResponse Object
     *
     * @param Zend_Http_Response $response
     */
    public static function createZendHttpResponse (Zend_Http_Response $response)
    {
        //	    	if (!self::validate($response)) {
        //	    		require_once 'Zend/Service/Exception.php';
        //	    		throw new Zend_Service_Exception('An error occurred sending request. Status code: ' . $response->getStatus());
        //	    	}
        $wsResponse = new WSResponse($response);
        return $wsResponse;
    }

    /**
     * Create Simple String WSResponse Object. It contains the data comming from the web service response. It will
     * not contians response headers.
     *
     * @param string $responseString
     * @throws WSException
     */
    public static function createHttpResponse ($responseString)
    {
        if (! self::validate($responseString)) {
            throw new WSException('An error occurred while creating Webservice Response object', WSException::$RESPONSE_VALIDATION_ERROR);
        }
        $wsResponse = new WSResponse($responseString);
        return $wsResponse;
    }

    /**
     * Validate the WSResponse object. return true if response data is correct and well formed. Return false
     * if response is not well formed or response return from web serive is false.
     *
     * @param Zend_Http_Response $response
     * @return boolean
     */
    private function validate ($response)
    {
        if (InQuireWebUtil::typeof($response) == 'string') {
        	if ($response == false)
        		return false;
        	else 
        		return true;
        } else
	        return $response->isError();
        //return true;
    }

    /**
     * Retrieve response body. It contians xml data.
     *
     * @return string
     */
    public function getReponseBody ()
    {
        if (InQuireWebUtil::typeof($this->response) == 'string')
            return $this->response; else
            return $this->response->getBody();
    }

    /**
     * Returns encoded response body.
     *
     * @return string
     */
    public function getEncodedReponse ()
    {
        if (InQuireWebUtil::typeof($this->response) == 'string')
            return urlencode($this->response); else
            return urlencode($this->response->getBody());
    }

    /**
     * This method return XML data in the form of SimpleXMLElement object. Creates light weight response object
     * easy process by php and smarty collections functions.
     *
     * @return SimpleXMLElement
     */
    public function getSimpleXMLResponse ()
    {
        $body = $this->getReponseBody();
        /**
         * Create SimpleXMLElement object from response text. Easy and lightweight
         * object to handle XML data inthe form of collections.
         */
        $xmlobject = simplexml_load_string($body);
        /**
         * Return SimpleXMLElement in response that will be easily manipulate
         * through PHP and Smarty collections iterators.
         */
        return $xmlobject;
    }

    /**
     * This method return well formed XML data that is used by any XSLT based style sheet to furhter process and
     * display it.
     *
     * @return string
     */
    public function getResponseForXSLT ()
    {
        $body = $this->getReponseBody();
        /**
         * Encode and response from ASCII characters to Unicode charactors
         */
        $response = utf8_encode($body);
        /**
         * Create SimpleXMLElement object from response text. Easy and lightweight
         * object to handle XML data inthe form of collections.
         */
        $xmlobject = new SimpleXMLElement($response);
        /**
         * Return pure XML data from SimpleXMLElement object for target XSTL file.
         */
        return $xmlobject->asXML();
    }

    /**
     * Returns the XML Response by using Zend_Rest api.
     *
     * @param string $WebService Location
     * @param string $Service Endpoint
     * @param array  $Paramters ( name => value )
     * @return Zend_Http_Response
     * @throws Zend_Service_Exception
     */
    public function getRestResponse ($url, $service, $params)
    {
        require_once 'Zend/Rest/Client.php';
        $rest = new Zend_Rest_Client($url);
        try {
            $result = $rest->restGet($service, $params);
        } catch (Zend_Rest_Client_Exception $e) {
            Logger::error("getSimpleXMLResponse Failed From SpexLiveService Client:" . $e->getMessage() . "\nDetail=>" . $e->getTraceAsString());
            throw new Zend_Service_Exception($e->getMessage());
        }
        return new SimpleXMLElement($result->getBody());
    }

    /**
     * Return the Status code for the Webservice response.
     *
     * @return int
     */
    public function getStatus ()
    {
        if (InQuireWebUtil::typeof($this->response) == 'string') {
            if ($this->response == false) {
                return WSException::$SERVICE_FAILED;
            } else {
            }
        } else {
            if ($this->response->isError()) {
                return WSException::$OTHER;
            } else {
            }
        }
    }

    /**
     * Return the Status Message for the Webservice response.
     *
     * @param Zend_Http_Response $response
     * @return string
     */
    public function getStatusMessage (Zend_Http_Response $response)
    {
        if (InQuireWebUtil::typeof($this->response) == 'string') {
            if ($this->response == false)
                return "Serviec Request Failed"; else
                return "Request Sucessful";
        } else
            return $response->getMessage();
    }
}
?>
